"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const postgres_interval_1 = __importDefault(require("postgres-interval"));
const column_interval_properties = ['years', 'months', 'days', 'hours', 'minutes', 'seconds', 'milliseconds'];
class PostgresColumnInterval {
    constructor(value) {
        this.interval = { years: 0, months: 0, days: 0, hours: 0, minutes: 0, seconds: 0, milliseconds: 0 };
        if (typeof value === 'string') {
            value = postgres_interval_1.default(value);
        }
        if (typeof value === 'object') {
            if ('interval' in value) {
                value = value.interval;
            }
            const interval = column_interval_properties.reduce((acc, key) => {
                if (value.hasOwnProperty(key) && value[key] !== null && value[key] !== undefined) {
                    acc[key] = value[key];
                }
                return acc;
            }, {});
            Object.assign(this.interval, interval);
        }
    }
    get years() {
        return Number(this.interval.years || 0);
    }
    get months() {
        return Number(this.interval.months || 0);
    }
    get days() {
        return Number(this.interval.days || 0);
    }
    get hours() {
        return Number(this.interval.hours || 0);
    }
    get minutes() {
        return Number(this.interval.minutes || 0);
    }
    get seconds() {
        return Number(this.interval.seconds || 0);
    }
    get milliseconds() {
        return Number(this.interval.milliseconds || 0);
    }
    valueOf() {
        return this.interval;
    }
    toString() {
        const agg = column_interval_properties.reduce((acc, key) => {
            if (key in this.interval && this.interval[key] !== undefined && this.interval[key] !== null && this.interval[key] > 0) {
                acc.push(`${this[key]} ${key}`);
            }
            return acc;
        }, []);
        return agg.join(' ') || '0';
    }
    toJSON() {
        return this.valueOf();
    }
    add(d) {
        if (d instanceof Date) {
            const dn = new Date(d.valueOf());
            dn.setMilliseconds(dn.getMilliseconds() + this.milliseconds);
            dn.setSeconds(dn.getSeconds() + this.seconds);
            dn.setMinutes(dn.getMinutes() + this.minutes);
            dn.setHours(dn.getHours() + this.hours);
            dn.setDate(dn.getDate() + this.days);
            dn.setMonth(dn.getMonth() + this.months);
            dn.setFullYear(dn.getFullYear() + this.years);
            return dn;
        }
        if (d instanceof PostgresColumnInterval) {
            const dn = new PostgresColumnInterval(this.valueOf());
            dn.interval = Object.keys(dn.interval).reduce((acc, el) => {
                acc[el] = dn[el] + d[el];
                return acc;
            }, dn.interval);
            return dn;
        }
        throw new Error('UNKNOWN ARG TYPE');
    }
    sub(d) {
        if (d instanceof Date) {
            const dn = new Date(d.valueOf());
            dn.setMilliseconds(dn.getMilliseconds() - this.milliseconds);
            dn.setSeconds(dn.getSeconds() - this.seconds);
            dn.setMinutes(dn.getMinutes() - this.minutes);
            dn.setHours(dn.getHours() - this.hours);
            dn.setDate(dn.getDate() - this.days);
            dn.setMonth(dn.getMonth() - this.months);
            dn.setFullYear(dn.getFullYear() - this.years);
            return dn;
        }
        if (d instanceof PostgresColumnInterval) {
            const dn = new PostgresColumnInterval(this.valueOf());
            dn.interval = Object.keys(dn.interval).reduce((acc, el) => {
                acc[el] = dn[el] - d[el];
                return acc;
            }, dn.interval);
            return dn;
        }
        throw new Error('UNKNOWN ARG TYPE');
    }
}
exports.default = PostgresColumnInterval;
exports.PostgresColumnInterval = PostgresColumnInterval;
//# sourceMappingURL=index.js.map